#include "stdafx.h"
#include "LockIn.h"

//--------------------------------------------------------------------------------
LockIn::LockIn(OscilloscopeHard const &hard)
  : m_hard(hard),
    m_succeeded(false)
{
  for (int i = 0; i <= MAX_CH_NUM; ++i)
  {
    m_amplitude[i] = 0.0f;
    m_phase[i] = 0.0f;
  }
}

bool LockIn::Run()
{
  m_succeeded = false;
  if (!m_hard.IsDDSEnabled()) return false;

  // Get DDS frequency, and check the length of sampled data.
  float ddsFreq;
  m_hard.GetDDSParam(&ddsFreq);
  double period = 1.0 / (double)ddsFreq;
  
  // Get oscilloscope data.
  int size = m_hard.GetDataSize();
  float const *data[MAX_CH_NUM];
  for (int i = 0; i < MAX_CH_NUM; ++i) data[i] = m_hard.GetVoltageData(i);

  float sampleFreq = OscilloscopeHard::SampleRate(m_hard.GetTimeDiv());
  double timeSpan = (double)size / sampleFreq;
  
  if (period > timeSpan) return false;  // Data length is too short.

  // Calculate lock in frequency.
  int countOfPeriod = wxRound(sampleFreq / ddsFreq);
  double lockInFreq = sampleFreq / countOfPeriod;
  m_freqError = std::abs(lockInFreq - ddsFreq);
  wxLogDebug(_T("m_freqError = %f"), m_freqError);

  // Analyze the center area of get data.
  int beginIdx = size / 2 - countOfPeriod / 2;
  int lastIdx = beginIdx + countOfPeriod - 1;

  double real[MAX_CH_NUM], imag[MAX_CH_NUM];
  for (int i = 0; i < MAX_CH_NUM; ++i) real[i] = imag[i] = 0.0;
  
  for (int i = beginIdx; i <= lastIdx; ++i)
  {
    int k = i - size / 2;
    double wnReal =  std::cos(2.0 * M_PI * k / countOfPeriod);
    double wnImag = -std::sin(2.0 * M_PI * k / countOfPeriod);
  
    for (int j = 0; j < MAX_CH_NUM; ++j)
    {
      real[j] += data[j][i] * wnReal;
      imag[j] += data[j][i] * wnImag;
    }
  }

  for (int i = 0; i < MAX_CH_NUM; ++i)
  {
    m_amplitude[i] = std::hypot(real[i], imag[i]) * 2.0 / countOfPeriod;
    m_phase[i] = std::atan2(imag[i], real[i]);
  }

  m_succeeded = true;
  return true;
}

float LockIn::GetAmplitude(int ch) const
{
  wxASSERT(ch >= CH1 && ch <= CH4);
  return m_amplitude[ch];
}

float LockIn::GetPhase(int ch) const
{
  wxASSERT(ch >= CH1 && ch <= CH4);
  return m_phase[ch];
}

//--------------------------------------------------------------------------------
