#include "stdafx.h"
#include "MainFrame.h"
#include "OscilloscopeParamDialog.h"
#include "DDSParamDialog.h"
#include "InputBox.h"
#include "UserInputRange.h"
#include "VoltDivDialog.h"
#include "SweepFrequencyDialog.h"

//--------------------------------------------------------------------------------
static int const TIMER_INTERVAL = 30;

// Status bar field definition.
static int const FIELD_DESC = 0;
static int const FIELD_CH1  = 1;
static int const FIELD_CH2  = 2;
static int const FIELD_CH3  = 3;
static int const FIELD_CH4  = 4;
static int const FIELD_HORZ = 5;
static int const FIELD_TRG  = 6;
static int const FIELD_DDS  = 7;
static int const FIELD_COUNT = 8;

static int const g_fieldWidths[FIELD_COUNT] =
{
  300,  // DESC
  80,   // CH1
  80,   // CH2
  80,   // CH3
  80,   // CH4
  50,   // HORZ
  150,  // TRG
  250   // DDS
};

//--------------------------------------------------------------------------------
MainFrame::MainFrame()
  : MainFrameDesign(NULL),
    m_lockIn(m_hard),
    m_dataRequested(false),
    m_hardGood(true),
    m_freqResLogScale(true)
{
  for (int i = CH1_2; i <= CH1_4; ++i) m_freqResChEnabled[i] = true;

  // Setup icon.
  SetIcon(wxICON(APP_ICON));

  // Setup status bar.
  m_statusBar->SetFieldsCount(FIELD_COUNT, g_fieldWidths);
  
  /*int styles[FIELD_COUNT];
  for (int i = 0; i < FIELD_COUNT; ++i) styles[i] = wxSB_SUNKEN;
  m_statusBar->SetStatusStyles(FIELD_COUNT, styles);*/

  UpdateStatusBar();

  // Setup table of grid.
  // Owner of the table object should be wxGrid to avoid access violation when frame is closed.
  // If the table object is hold as MainFrame member, the table seems to be refered after destruction
  // and access error occurs.
  m_table = new SweepTable();
  m_gridSweep->SetTable(m_table, true);
  m_gridSweep->AutoSizeColumns();

  // Setup timer.
  m_timer.SetOwner(this);

  // Setup oscilloscope hardware.
  m_hardGood = m_hard.Init() ? true : false;
  if (m_hardGood)
  {
    m_timer.Start(TIMER_INTERVAL);
  }
  else
  {
    wxMessageBox(m_hard.GetLastErrorMessage());
  }

  // Setup sweep controller.
  m_ctrl = new SweepControl(m_hard, m_lockIn, *m_table);

  Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::OnTimer));
}

MainFrame::~MainFrame()
{
  Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::OnTimer));
  delete m_ctrl;
}

void MainFrame::OnTimer(wxTimerEvent &event)
{
  if (m_dataRequested)
  {
    if (!m_hard.StartCollect())
    {
      wxMessageBox(m_hard.GetLastErrorMessage());
      m_timer.Stop();
    }
    m_dataRequested = false;
  }

  if (m_hard.CheckCollected())
  {
    m_hard.ReadData();

    // Lock in.
    if (m_lockIn.Run())
    {
      wxLogDebug(_T("CH1:%f[V],%f[rad], CH2:%f[V],%f[rad], CH3:%f[V],%f[rad], CH4:%f[V],%f[rad]"),
                 m_lockIn.GetAmplitude(CH1), m_lockIn.GetPhase(CH1),
                 m_lockIn.GetAmplitude(CH2), m_lockIn.GetPhase(CH2),
                 m_lockIn.GetAmplitude(CH3), m_lockIn.GetPhase(CH3),
                 m_lockIn.GetAmplitude(CH4), m_lockIn.GetPhase(CH4));
    }

    // Sweep control process.
    int notify = m_ctrl->OnOscilloscopeReadData();
    if ((notify & SweepControl::NOTIFY_ERROR) != 0)
    {
      wxMessageBox(m_ctrl->GetLastErrorMessage());
    }
    if ((notify & SweepControl::NOTIFY_TABLE_UPDATED) != 0)
    {
      //m_gridSweep->Refresh();
      m_panelFreqResponse->Refresh();
    }
    if ((notify & SweepControl::NOTIFY_SWEEP_COMPLETED) != 0)
    {
      wxMessageBox(_("Sweep is COMPLETED!"));
    }

    // Request to update display.
    m_panelWaveform->Refresh(false);
    m_dataRequested = true;
  }
  else
  {
    m_dataRequested = false;
  }
}

void MainFrame::PanelWaveform_OnPaint(wxPaintEvent &event)
{
  wxWindow *window = m_panelWaveform;
  wxRect rect = window->GetClientRect();
  if (0 == rect.width || 0 == rect.height)
  {
    return;
  }

  //wxPaintDC dc(window);
  
  RECT winrect;
  winrect.top = rect.y;
  winrect.left = rect.x;
  winrect.bottom = rect.y + rect.height;
  winrect.right = rect.x + rect.width;

  // Get the handle of the wxWindow
  HWND hwnd = static_cast<HWND>(window->GetHandle());

  // Now get the native windows DC for this window
  HDC hdc = ::GetDC(hwnd);

  // Create a double buffer for blitting to
  // the screen to prevent screen flicker. Pass
  // the double buffer to cairo and blit it
  // in the paint routine.
  HDC dcbuffer = ::CreateCompatibleDC(hdc);
  HBITMAP hbuffer = ::CreateCompatibleBitmap(hdc, rect.width, rect.height);
  ::SelectObject(dcbuffer, hbuffer); 

  // Draw wave data.
  if (m_hardGood)
  {
    m_hard.Draw(dcbuffer, winrect);

    // Draw text.
    if (m_lockIn.IsSucceeded())
    {
      ::SetBkMode(dcbuffer, TRANSPARENT);

      wxString str1 = wxString::Format(_T("CH1 %10.6fV, %6.1fdeg"), m_lockIn.GetAmplitude(CH1), m_lockIn.GetPhase(CH1) * 180.0f / M_PI);
      wxString str2 = wxString::Format(_T("CH2 %10.6fV, %6.1fdeg"), m_lockIn.GetAmplitude(CH2), m_lockIn.GetPhase(CH2) * 180.0f / M_PI);
      wxString str3 = wxString::Format(_T("CH3 %10.6fV, %6.1fdeg"), m_lockIn.GetAmplitude(CH3), m_lockIn.GetPhase(CH3) * 180.0f / M_PI);
      wxString str4 = wxString::Format(_T("CH4 %10.6fV, %6.1fdeg"), m_lockIn.GetAmplitude(CH4), m_lockIn.GetPhase(CH4) * 180.0f / M_PI);

      m_hard.DrawText(dcbuffer, CH1, str1);
      m_hard.DrawText(dcbuffer, CH2, str2);
      m_hard.DrawText(dcbuffer, CH3, str3);
      m_hard.DrawText(dcbuffer, CH4, str4);
    }
  }

  // Now blit the object to the screen
  ::BitBlt(hdc, 0, 0, rect.width, rect.height, dcbuffer, 0, 0, SRCCOPY);

  ::DeleteDC(dcbuffer);
  ::DeleteObject(hbuffer);

  // Because we called ::GetDC make sure we release the handle
  // back to the system or we'll have a memory leak.
  ::ReleaseDC(hwnd, hdc);  
}

void MainFrame::PanelWaveform_OnSize(wxSizeEvent &event)
{
}

void MainFrame::PanelFreqResponse_OnPaint(wxPaintEvent &event)
{
  wxWindow *window = m_panelFreqResponse;
  wxRect rect = window->GetClientRect();
  if (0 == rect.width || 0 == rect.height)
  {
    return;
  }

  wxPaintDC dc(window);

  SweepTableDrawOption option;
  option.drawCurrentFreq = m_ctrl->IsSweeping();
  if (option.drawCurrentFreq)
  {
    int row = m_ctrl->GetCurrentRow();
    option.currentFreq = (*m_table)[row].ddsFreq;
  }
  m_table->Draw(dc, rect, m_freqResChEnabled, m_freqResLogScale, option);
}

void MainFrame::PanelFreqResponse_OnSize(wxSizeEvent &event)
{
  m_panelFreqResponse->Refresh(false);
}

//--------------------------------------------------------------------------------
void MainFrame::GridSweep_OnGridLabelLeftDClick(wxGridEvent &event)
{
  int row = event.GetRow();
  int col = event.GetCol();

  wxLogDebug(_T("row = %d, col = %d"), row, col);

  if (col >= 0 && -1 == row)
  {
    // On right click over row label.
    switch (col)
    {
    case COL_DDS_FREQ:
      MenuItemSetupFreq_OnMenuSelection(wxCommandEvent());
      break;
    case COL_DDS_AMP:
      MenuItemSetupAmp_OnMenuSelection(wxCommandEvent());
      break;
    case COL_DDS_OFFSET:
      MenuItemSetupOffset_OnMenuSelection(wxCommandEvent());
      break;
    case COL_VOLT_DIV_CH1:
    case COL_VOLT_DIV_CH2:
    case COL_VOLT_DIV_CH3:
    case COL_VOLT_DIV_CH4:
      MenuItemSetupVoltDiv_OnMenuSelection(wxCommandEvent());
      break;
    case COL_AVERAGE:
      MenuItemSetupAverage_OnMenuSelection(wxCommandEvent());
      break;
    }
  }
}

void MainFrame::GridSweep_OnGridLabelRightClick(wxGridEvent &event)
{
  int row = event.GetRow();
  int col = event.GetCol();

  wxLogDebug(_T("row = %d, col = %d"), row, col);

  if (row >= 0 && -1 == col)
  {
    // On right click over row label.
    m_gridSweep->SelectRow(row);
    wxString itemLabel = wxString::Format(_("Apply Sweep Param No. %d"), row + 1);
    m_menuItemApplyCurrentSweepParam->SetItemLabel(itemLabel);
    PopupMenu(m_menuPopupOverRowLabel);
  }
}

//--------------------------------------------------------------------------------
void MainFrame::MenuItemApplyCurrentSweepParam(wxCommandEvent &event)
{
  wxLogDebug(_("Apply Current Sweep Param"));
  
  if (!CheckHardGood()) return;

  if (m_ctrl->IsSweeping())
  {
    wxMessageBox(_("Now sweeping! Please execute after sweep is stopped!"));
  }
  else
  {
    wxArrayInt rows = m_gridSweep->GetSelectedRows();
    if (!rows.empty())
    {
      m_ctrl->ApplySweepCondition(rows[0]);
      UpdateStatusBar();
    }
  }
}

void MainFrame::MenuItemLoad_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();
  wxString fileName
    = wxFileSelector(_("Load Sweep Settings"), _T(""), _T("Default.csv"),
                     _T("csv"), _T("CSV files (*.csv)|*.csv"), wxFD_OPEN | wxFD_FILE_MUST_EXIST, this);
  if (!fileName.empty())
  {
    if (!m_table->LoadFromFile(fileName))
    {
      wxMessageBox(_("Failed to read file!"));
    }
  }
  AfterUserAction();
}

void MainFrame::MenuItemSave_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();
  wxString fileName
    = wxFileSelector(_("Save Sweep Settings"), _T(""), _T("Default.csv"),
                     _T("csv"), _T("CSV files (*.csv)|*.csv"), wxFD_SAVE | wxFD_OVERWRITE_PROMPT, this);
  if (!fileName.empty())
  {
    m_table->SaveToFile(fileName);
  }
  AfterUserAction();
}

void MainFrame::MenuItemSaveWaveform_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  wxDateTime now = wxDateTime::Now();
  //wxString fileName = _T("Waveform") + now.Format(_T("%y%m%d-%H%M%S")) + _T(".csv");
  wxString fileName
    = wxFileSelector(_("Save Waveform"), _T(""), _T("Default.csv"),
                     _T("csv"), _T("CSV files (*.csv)|*.csv"), wxFD_SAVE | wxFD_OVERWRITE_PROMPT, this);
  if (!fileName.empty())
  {
    m_hard.SaveWaveform(fileName);
  }
  AfterUserAction();
}

void MainFrame::MenuItemExit_OnMenuSelection(wxCommandEvent &event)
{
  Close();
}

void MainFrame::MenuItemSetOscParam_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  OscilloscopeParamDialog dialog(this, m_hard);
  dialog.ShowModal();
  AfterUserAction();
}

void MainFrame::MenuItemSetDDSParam_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  DDSParamDialog dialog(this, m_hard);
  dialog.ShowModal();
  AfterUserAction();
}

void MainFrame::MenuItemEnableDDS_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  BOOL enabled = m_hard.IsDDSEnabled();
  enabled = !enabled;
  m_hard.EnableDDS(enabled);
  AfterUserAction();
}

void MainFrame::MenuItemEnableDDS_OnUpdateUI(wxUpdateUIEvent &event)
{
  event.Check(m_hard.IsDDSEnabled() ? true : false);
}

void MainFrame::MenuItemStart_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  if (m_ctrl->IsSweeping())
  {
    wxMessageBox(_("Already sweeping is executed!"));
  }
  else
  {
    m_ctrl->StartSweep();
  }
  AfterUserAction();
}

void MainFrame::MenuItemAbort_OnMenuSelection(wxCommandEvent &event)
{
  if (!CheckHardGood()) return;
  BeforeUserAction();
  if (m_ctrl->IsSweeping())
  {
    m_ctrl->AbortSweep();
  }
  AfterUserAction();
}

void MainFrame::MenuItemSetSweepPoints_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();

  int points = m_table->GetSize();
  wxString prompt = wxString::Format(_("Sweep Points (%d - %d):"), USER_MIN_SWEEP_POINTS, USER_MAX_SWEEP_POINTS);
  if (wxID_OK == InputBoxInt(points, prompt, _("Sweep Setting"), this, USER_MIN_SWEEP_POINTS, USER_MAX_SWEEP_POINTS))
  {
    m_table->Resize(points);
  }

  AfterUserAction();
}

void MainFrame::MenuItemSetupAverage_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();

  int average = (*m_table)[m_table->GetSize() - 1].average;   // Default value is the last data.
  wxString prompt = wxString::Format(_("Count of average (1 - %d):"), USER_MAX_AVERAGE);
  if (wxID_OK == InputBoxInt(average, prompt, _("Sweep Setting"), this, 1, USER_MAX_AVERAGE))
  {
    m_table->FillAverage(average);
  }

  AfterUserAction();
}

void MainFrame::MenuItemSetupFreq_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();
  SweepFrequencyDialog dialog(this, *m_table);
  dialog.ShowModal();
  AfterUserAction();
}

void MainFrame::MenuItemSetupAmp_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();

  int ddsAmp = (*m_table)[m_table->GetSize() - 1].ddsAmp;       // Default value is current setting.
  wxString prompt = wxString::Format(_("DDS Amplitude (0 - %d) [mV]:"), USER_MAX_DDS_AMP);
  if (wxID_OK == InputBoxInt(ddsAmp, prompt, _("Sweep Setting"), this, 0, USER_MAX_DDS_AMP))
  {
    m_table->FillDDSAmplitude(ddsAmp);
  }

  AfterUserAction();
}

void MainFrame::MenuItemSetupOffset_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();

  int ddsOffset = (*m_table)[m_table->GetSize() - 1].ddsOffset; // Default value is current setting.
  wxString prompt = wxString::Format(_("DDS Offset (%d - %d) [mV]:"), -USER_MAX_DDS_OFFSET, USER_MAX_DDS_OFFSET);
  if (wxID_OK == InputBoxInt(ddsOffset, prompt, _("Sweep Setting"), this, -USER_MAX_DDS_OFFSET, USER_MAX_DDS_OFFSET))
  {
    m_table->FillDDSOffset(ddsOffset);
  }

  AfterUserAction();
}

void MainFrame::MenuItemSetupVoltDiv_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();
  VoltDivDialog dialog(this, *m_table);
  dialog.ShowModal();
  AfterUserAction();
}

void MainFrame::MenuItemOptimizeVoltDiv_OnMenuSelection(wxCommandEvent &event)
{
  BeforeUserAction();

  int ans = wxMessageBox(_("Do you use 50ohm termination for DDS output?"), _("Optimize Volt/DIV"), wxYES_NO, this);
  bool term50Ohm = (wxYES == ans);
  m_table->OptimizeVoltDiv(term50Ohm);

  AfterUserAction();
}

void MainFrame::MenuItemShowCh1_2_OnMenuSelection(wxCommandEvent &event)
{
  m_freqResChEnabled[CH1_2] = !m_freqResChEnabled[CH1_2];
  m_panelFreqResponse->Refresh(false);
}

void MainFrame::MenuItemShowCh1_2_OnUpdateUI(wxUpdateUIEvent &event)
{
  event.Check(m_freqResChEnabled[CH1_2]);
}

void MainFrame::MenuItemShowCh1_3_OnMenuSelection(wxCommandEvent &event)
{
  m_freqResChEnabled[CH1_3] = !m_freqResChEnabled[CH1_3];
  m_panelFreqResponse->Refresh(false);
}

void MainFrame::MenuItemShowCh1_3_OnUpdateUI(wxUpdateUIEvent &event)
{
  event.Check(m_freqResChEnabled[CH1_3]);
}

void MainFrame::MenuItemShowCh1_4_OnMenuSelection(wxCommandEvent &event)
{
  m_freqResChEnabled[CH1_4] = !m_freqResChEnabled[CH1_4];
  m_panelFreqResponse->Refresh(false);
}

void MainFrame::MenuItemShowCh1_4_OnUpdateUI(wxUpdateUIEvent &event)
{
  event.Check(m_freqResChEnabled[CH1_4]);
}

void MainFrame::MenuItemLogScale_OnMenuSelection(wxCommandEvent &event)
{
  m_freqResLogScale = !m_freqResLogScale;
  m_panelFreqResponse->Refresh(false);
}

void MainFrame::MenuItemLogScale_OnUpdateUI(wxUpdateUIEvent &event)
{
  event.Check(m_freqResLogScale);
}

void MainFrame::MenuItemAbout_OnMenuSelection(wxCommandEvent &event)
{
  wxAboutDialogInfo aboutInfo;
  aboutInfo.SetName(_("Frequency Response Analyzer for HT6004BX"));
  aboutInfo.SetVersion(_("190622"));
  aboutInfo.SetCopyright(_("Copyright (C) 2019 Euphorie All Rights Reserved."));
  //aboutInfo.AddDeveloper(_("Euphorie"));
  aboutInfo.SetIcon(wxICON(APP_ICON));
  //aboutInfo.SetLicence(_("BSD-3-Clause"));
  aboutInfo.SetDescription(_("License: BSD-3-Clause"));

  wxAboutBox(aboutInfo);
}

//--------------------------------------------------------------------------------
void MainFrame::UpdateStatusBar()
{
  /// @todo implement more effective code
  wxArrayString channelTexts = OscilloscopeHard::GetChannelTexts();
  //wxArrayString timeDivTexts = OscilloscopeHard::GetTimeDivTexts();
  //wxArrayString voltDivTexts = OscilloscopeHard::GetVoltDivTexts();
  wxArrayString trgSlopeTexts = OscilloscopeHard::GetTriggerSlopeTexts();
  wxArrayString couplingTexts = OscilloscopeHard::GetCouplingTexts();

  //wxString ch1 = _T("CH1:") + couplingTexts[m_hard.GetCoupling(CH1)] + _T(",") + voltDivTexts[m_hard.GetVoltDiv(CH1)];
  //wxString ch2 = _T("CH2:") + couplingTexts[m_hard.GetCoupling(CH2)] + _T(",") + voltDivTexts[m_hard.GetVoltDiv(CH2)];
  //wxString ch3 = _T("CH3:") + couplingTexts[m_hard.GetCoupling(CH3)] + _T(",") + voltDivTexts[m_hard.GetVoltDiv(CH3)];
  //wxString ch4 = _T("CH4:") + couplingTexts[m_hard.GetCoupling(CH4)] + _T(",") + voltDivTexts[m_hard.GetVoltDiv(CH4)];
  wxString ch1 = _T("CH1:") + couplingTexts[m_hard.GetCoupling(CH1)] + _T(",") + OscilloscopeHard::VoltDivToString(m_hard.GetVoltDiv(CH1));
  wxString ch2 = _T("CH2:") + couplingTexts[m_hard.GetCoupling(CH2)] + _T(",") + OscilloscopeHard::VoltDivToString(m_hard.GetVoltDiv(CH2));
  wxString ch3 = _T("CH3:") + couplingTexts[m_hard.GetCoupling(CH3)] + _T(",") + OscilloscopeHard::VoltDivToString(m_hard.GetVoltDiv(CH3));
  wxString ch4 = _T("CH4:") + couplingTexts[m_hard.GetCoupling(CH4)] + _T(",") + OscilloscopeHard::VoltDivToString(m_hard.GetVoltDiv(CH4));

  //wxString horz = timeDivTexts[m_hard.GetTimeDiv()];
  wxString horz = OscilloscopeHard::TimeDivToString(m_hard.GetTimeDiv());

  WORD ch;
  USHORT slope;
  SHORT level;
  m_hard.GetTriggerParam(&ch, &slope, &level);
  wxString trg = wxString::Format(_T("TRG:%s,%s,%dmV"), channelTexts[ch], trgSlopeTexts[slope], level);

  float freq;
  USHORT amp;
  SHORT offset;
  m_hard.GetDDSParam(&freq, &amp, &offset);
  wxString dds = wxString::Format(_T("DDS:%.3fHz,amp=%dmV,offset=%dmV"), freq, amp, offset);

  m_statusBar->SetStatusText(ch1, FIELD_CH1);
  m_statusBar->SetStatusText(ch2, FIELD_CH2);
  m_statusBar->SetStatusText(ch3, FIELD_CH3);
  m_statusBar->SetStatusText(ch4, FIELD_CH4);
  m_statusBar->SetStatusText(horz, FIELD_HORZ);
  m_statusBar->SetStatusText(trg, FIELD_TRG);
  m_statusBar->SetStatusText(dds, FIELD_DDS);
}

bool MainFrame::CheckHardGood()
{
  if (!m_hardGood)
  {
    wxMessageBox(_("Oscilloscope is not available!"));
  }
  return m_hardGood;
}

void MainFrame::BeforeUserAction()
{
  m_timer.Stop();
}

void MainFrame::AfterUserAction()
{
  UpdateStatusBar();
  m_panelFreqResponse->Refresh(false);
  if (m_hardGood)
  {
    m_timer.Start();
  }
}

//--------------------------------------------------------------------------------
