#pragma once

//--------------------------------------------------------------------------------
/**
 * Oscilloscope Hardware Management Class.
 */
class OscilloscopeHard  
{
public:
	OscilloscopeHard();
	virtual ~OscilloscopeHard();

  /**
   * Initialize.
   */
	BOOL Init();

  /**
   * Set timebase.
   */
  void SetTimeDiv(USHORT nTimeDiv);

  /**
   * Get timebase.
   */
  USHORT GetTimeDiv() const { return m_nTimeDIV; }

  /**
   * Set trigger parameters.
   * @param nSource CH1 - CH4.
   * @param nSlope  RISE or FALL.
   * @param nLevel  Trigger level [mV].
   */
  void SetTriggerParam(WORD nSource, USHORT nSlope, SHORT nLevel); 

  /**
   * Get trigger parameters.
   */
  void GetTriggerParam(WORD *pnSource, USHORT *pnSlope, SHORT *pnLevel) const;

  /**
   * Set voltage base.
   */
  void SetVoltDiv(USHORT nCH, WORD nCHVoltDIV);

  /**
   * Get voltage base.
   */
  WORD GetVoltDiv(USHORT nCH) const;

  /**
   * Set coupling.
   */
  void SetCoupling(USHORT nCH, WORD nCHCoupling);

  /**
   * Get coupling.
   */
  WORD GetCoupling(USHORT nCH) const;

  /**
   * Apply measurement condition to the hardware
   */
  BOOL ApplyCondition();

  /**
   * Start to collect new data.
   */
  BOOL StartCollect();

  /**
   * Check whether collecting data is completed or not.
   */
  BOOL CheckCollected();

  /**
   * Read the collected data.
   */
	BOOL ReadData();

  /**
   * Get the bytes of data.
   */
  ULONG GetDataSize() const;

  /**
   * Get the pointer of data.
   */
  SHORT const *GetDisplayData(USHORT nCH) const;

  /**
   * Get voltage wave data.
   */
  float const *GetVoltageData(USHORT nCH) const;

  /**
   * Draw to device context.
   */
  void Draw(HDC hDC, RECT const &Rect);
  
  /**
   * Draw text related with each channel.
   */
  void DrawText(HDC hDC, USHORT nCH, wxString text);

  /**
   * Save waveform to CSV.
   */
  BOOL SaveWaveform(wxString const &fileName) const;

  /**
   * Setup DDS parameters.
   * @param fFrequency  Frequency [Hz].
   * @param nAmp        Amplitude [mV].
   * @param nOffset     Offset [mV].
   */
  BOOL SetDDSParam(float fFrequency, USHORT nAmp, SHORT nOffset);

  /**
   * Get DDS parameters.
   */
  void GetDDSParam(float *pfFrequency = NULL, USHORT *pnAmp = NULL, SHORT *pnOffset = NULL) const;

  /**
   * ON/OFF DDS.
   */
  BOOL EnableDDS(BOOL bEnabled);

  /**
   * Check ON/OFF of DDS.
   */
  BOOL IsDDSEnabled() const { return m_bDDSEnabled; }

  /**
   * Get the message of last error.
   */
  wxString const &GetLastErrorMessage() const { return m_err; }

  // Conversion functions.
  static wxString TimeDivToString(USHORT nTimeDiv);
  static USHORT TimeDivFromString(wxString const &str);
  static wxString VoltDivToString(WORD nVoltDiv);
  static WORD VoltDivFromString(wxString const &str);

  static wxArrayString GetChannelTexts();
  static wxArrayString GetTimeDivTexts();
  static wxArrayString GetVoltDivTexts();
  static wxArrayString GetTriggerSlopeTexts();
  static wxArrayString GetCouplingTexts();
  static float SampleRate(USHORT nTimeDIV);
  static float ValueOfTimeDiv(USHORT nTimeDIV);
  static float ValueOfVoltDiv(WORD nVoltDIV);

private:
  void ClearError() const { m_err.clear(); }
  void SetError(wxString const &err) const { m_err = err; }

  void MakeControlData(CONTROLDATA &stControl, RELAYCONTROL &stRelayControl);
  void SourceToDisplay(USHORT* pData, ULONG nDataLen, USHORT nCH);
  void SourceToVoltage(USHORT* pData, ULONG nDataLen, USHORT nCH);

  void DrawWave(HDC hDC, RECT const &Rect);
  void DrawWaveInYT(HDC hDC, RECT const &Rect, USHORT nCH);
  void DrawGrid(HDC hDC, RECT const &Rect);

	USHORT m_nDeviceIndex;            // Detected device index.
	SHORT* m_pDispData[MAX_CH_NUM];   // Read data for display (0-255).
  float* m_pVoltData[MAX_CH_NUM];   // Read data for calculate [V].

  SHORT m_nTriggerMode;             // Trigger mode (Edge detecting, etc...)
	USHORT m_nTriggerSweep;           // AUTO or NORMAL.
	USHORT m_nTriggerSlope;           // Rising or Falling.
  WORD m_nTriggerSource;            // Trigger source.
  SHORT m_nTriggerLevel;            // Trigger level [mV].

  WORD m_nCHVoltDIV[MAX_CH_NUM];    // Voltage range of each channel.
  WORD m_nCHCoupling[MAX_CH_NUM];   // Coupling of each channel.

	USHORT m_nTimeDIV;                // Time axis scaling.
	USHORT m_nYTFormat;               // Axes format of plotting.
	USHORT m_nLeverPos[MAX_CH_NUM];   // Position of each channel.
	COLORREF m_clrRGB[MAX_CH_NUM];    // Color or each channel.

	USHORT m_nReadOK;                 // Whether reading is succeeded or not.
  BOOL m_bInit;                     // Whether device is initialized or not.

  float m_fDDSFrequency;            // DDS frequency.
  USHORT m_nDDSAmp;                 // DDS amplitude.
  SHORT m_nDDSOffset;               // DDS offset.
  BOOL m_bDDSEnabled;               // DDS enabled.

  mutable wxString m_err;           // Error message.
};

//--------------------------------------------------------------------------------
