#include "stdafx.h"
#include "SweepControl.h"

//------------------------------------------------------------------------
SweepControl::SweepControl(OscilloscopeHard &hard, LockIn &lockIn, SweepTable &table)
  : m_hard(hard),
    m_lockIn(lockIn),
    m_table(table),
    m_sweeping(false),
    m_rowCounter(0),
    m_aveCounter(0),
    m_waitCounter(0)
{
  for (int i = CH1_2; i <= CH1_4; ++i) m_realSum[i] = m_imagSum[i] = 0.0f;
}

bool SweepControl::ApplySweepCondition(int row)
{
  wxASSERT(row >= 0 && row < m_table.GetSize());
  ClearError();

  // Set DDS parameter.
  Measurement const &meas = m_table[row];
  if (!m_hard.SetDDSParam(meas.ddsFreq, meas.ddsAmp, meas.ddsOffset))
  {
    SetError(m_hard.GetLastErrorMessage());
    return false;
  }

  // Set Oscilloscope parameter.
  m_hard.SetTimeDiv(meas.timeDiv);
  for (int i = CH1; i <= CH4; ++i) m_hard.SetVoltDiv(i, meas.voltDiv[i]);
  if (!m_hard.ApplyCondition())
  {
    SetError(m_hard.GetLastErrorMessage());
    return false;
  }

  return true;
}

void SweepControl::StartSweep()
{
  wxASSERT(!m_sweeping);

  if (!m_sweeping && m_table.GetSize() > 0)
  {
    m_rowCounter = 0;
    NewCondition(0);
    m_hard.EnableDDS(TRUE);
    m_sweeping = true;
  }  
}

void SweepControl::AbortSweep()
{
  wxASSERT(m_sweeping);
  m_sweeping = false;
}

int SweepControl::OnOscilloscopeReadData()
{
  ClearError();
  int notify = NOTIFY_NONE;

  if (m_sweeping)
  {
    // Ignore data if m_waitCouner > 0.
    if (m_waitCounter > 0)
    {
      m_waitCounter--;
      return notify;
    }

    // Calculate gain and phase.
    if (!m_lockIn.IsSucceeded())
    {
      SetError(_("Lock in is failed!"));
      notify |= NOTIFY_ERROR;
      return notify;
    }

#if 0
    // Save waveform data (debug). 
    wxDateTime now = wxDateTime::Now();
    wxString fileName = wxString::Format(_T("Sweep-%03d-%02d_%s.csv"), m_rowCounter, m_aveCounter, now.Format(_T("%y%m%d-%H%M%S")));
    m_hard.SaveWaveform(fileName);
#endif

    // Calculate gain and phase.
    float ampl1  = m_lockIn.GetAmplitude(CH1);
    if (ampl1 < 0.000001f) ampl1 = 0.000001f;           // Amplitude make larger than 1uV. 
    float phase1 = m_lockIn.GetPhase(CH1);

    for (int i = CH1_2; i <= CH1_4; ++i)
    {
      float ampl  = m_lockIn.GetAmplitude(i + 1);
      if (ampl < 0.000001f) ampl = 0.000001f;           // Amplitude make larger than 1uV. 
      float gain  = ampl / ampl1;
      float phase = m_lockIn.GetPhase(i + 1) - phase1;
      m_realSum[i] += gain * std::cos(phase);
      m_imagSum[i] += gain * std::sin(phase);
    }

    // Check average counter.
    m_aveCounter ++;
    int average = m_table[m_rowCounter].average;
    if (m_aveCounter == average)
    {
      // Averaging is completed.
      for (int i = CH1_2; i <= CH1_4; ++i)
      {
        float real = m_realSum[i] / average;
        float imag = m_imagSum[i] / average;
        m_table[m_rowCounter].measGain[i] = 20.0f * std::log10(std::hypot(real, imag));
        m_table[m_rowCounter].measPhase[i] = std::atan2(imag, real) * 180.0f / M_PI;
      }
      notify |= NOTIFY_TABLE_UPDATED;
      m_table.NotifyUpdate();

      // Check row counter.
      m_rowCounter ++;
      if (m_rowCounter == m_table.GetSize())
      {
        // Sweep is completed.
        notify |= NOTIFY_SWEEP_COMPLETED;
        m_sweeping = false;
      }
      else
      {
        NewCondition(m_rowCounter);
      }
    }
  }

  return notify;
}

void SweepControl::NewCondition(int row)
{
  m_aveCounter = 0;
  m_waitCounter = 1;  // Ignore next read data.
  for (int i = CH1_2; i <= CH1_4; ++i) m_realSum[i] = m_imagSum[i] = 0.0f;
  ApplySweepCondition(row);
}

//------------------------------------------------------------------------
