#include "stdafx.h"
#include "SweepTable.h"
#include "OscilloscopeHard.h"

//------------------------------------------------------------------------
static wxString const g_csvFileHeader =
 _T("\"DDS Freq.[Hz]\",")
 _T("\"DDS Amp.[mV]\",")
 _T("\"DDS Offset[mV]\",")
 _T("\"Time/DIV\",")
 _T("\"Volt1/DIV\",")
 _T("\"Volt2/DIV\",")
 _T("\"Volt3/DIV\",")
 _T("\"Volt4/DIV\",")
 _T("\"Average\",")
 _T("\"|G12|[dB]\",")
 _T("\"|G13|[dB]\",")
 _T("\"|G14|[dB]\",")
 _T("\"arg(G12)[deg]\",")
 _T("\"arg(G13)[deg]\",")
 _T("\"arg(G14)[deg]\"");

//------------------------------------------------------------------------
static wxString FormatEng(float value)
{
  wxString format = _T("%.0f");
  
  if      (value >= 1.0e+9f)  return wxString::Format(format + _T("G"), value * 1.0e-9);
  else if (value >= 1.0e+6f)  return wxString::Format(format + _T("M"), value * 1.0e-6);
  else if (value >= 1.0e+3f)  return wxString::Format(format + _T("k"), value * 1.0e-3);
  else if (value >= 1.0f)     return wxString::Format(format,           value);
  else if (value >= 1.0e-3f)  return wxString::Format(format + _T("m"), value * 1.0e+3);
  else if (value >= 1.0e-6f)  return wxString::Format(format + _T("u"), value * 1.0e+6);
  else if (value >= 1.0e-9f)  return wxString::Format(format + _T("n"), value * 1.0e+9);
  //else                        return wxString::Format(format + _T("p"), value * 1.0e+12);
  else                        return _T("0");
}

//------------------------------------------------------------------------
Measurement::Measurement()
  : ddsFreq(10000.0f),
    ddsAmp(100),
    ddsOffset(0),
    timeDiv(12),        // Time base = 20us, Sampling rate = 12.5MSa
    average(1)
{
  for (int i = 0; i < MAX_CH_NUM; ++i)
  {
    voltDiv[i] = 5;     // Voltage base = 100mV, range = 800mV
  }

  for (int i = 0; i < MAX_CH_NUM - 1; ++i)
  {
    measGain[i] = 0.0f;
    measPhase[i] = 0.0f;
  }
}

//------------------------------------------------------------------------
SweepTable::SweepTable()
{
  Resize(21);  // Test
  FillFrequency(1000.0f, 100000.f, true, true);

  if (this->CanHaveAttributes())  // By default, the table automatically creates a wxGridCellAttrProvider 
  {
    // COL_DDS_FREQ
    wxGridCellAttr *attrDDSFreq = new wxGridCellAttr;
    attrDDSFreq->SetEditor(new wxGridCellFloatEditor(-1, 3));
    this->SetColAttr(attrDDSFreq, COL_DDS_FREQ);

    // COL_DDS_AMP
    wxGridCellAttr *attrDDSAmp = new wxGridCellAttr;
    attrDDSAmp->SetEditor(new wxGridCellNumberEditor(-1, -1));
    this->SetColAttr(attrDDSAmp, COL_DDS_AMP);

    // COL_DDS_OFFSET
    wxGridCellAttr *attrDDSOffset = new wxGridCellAttr;
    attrDDSOffset->SetEditor(new wxGridCellNumberEditor(-1, -1));
    this->SetColAttr(attrDDSOffset, COL_DDS_OFFSET);

    // COL_TIME_DIV
    wxGridCellAttr *attrTimeDiv = new wxGridCellAttr;
    attrTimeDiv->SetEditor(new wxGridCellChoiceEditor(OscilloscopeHard::GetTimeDivTexts()));
    this->SetColAttr(attrTimeDiv, COL_TIME_DIV);

    // COL_VOLT_DIV_CHx
    wxGridCellAttr *attrVoltDiv = new wxGridCellAttr;
    attrVoltDiv->SetEditor(new wxGridCellChoiceEditor(OscilloscopeHard::GetVoltDivTexts()));
    this->SetColAttr(attrVoltDiv, COL_VOLT_DIV_CH1);
    this->SetColAttr(attrVoltDiv, COL_VOLT_DIV_CH2);  attrVoltDiv->IncRef();
    this->SetColAttr(attrVoltDiv, COL_VOLT_DIV_CH3);  attrVoltDiv->IncRef();
    this->SetColAttr(attrVoltDiv, COL_VOLT_DIV_CH4);  attrVoltDiv->IncRef();

    // COL_AVERAGE
    wxGridCellAttr *attrAverage = new wxGridCellAttr;
    attrAverage->SetEditor(new wxGridCellNumberEditor(-1, -1));
    this->SetColAttr(attrAverage, COL_AVERAGE);

    // COL_MEAS_AMP_CHx, OL_MEAS_PHASE_CHx
    wxGridCellAttr *attrReadOnly = new wxGridCellAttr;
    attrReadOnly->SetReadOnly();
    attrReadOnly->SetBackgroundColour(wxColor(0x000033));
    this->SetColAttr(attrReadOnly, COL_MEAS_GAIN_CH1_2);
    this->SetColAttr(attrReadOnly, COL_MEAS_GAIN_CH1_3);  attrReadOnly->IncRef();
    this->SetColAttr(attrReadOnly, COL_MEAS_GAIN_CH1_4);  attrReadOnly->IncRef();
    this->SetColAttr(attrReadOnly, COL_MEAS_PHASE_CH1_2); attrReadOnly->IncRef();
    this->SetColAttr(attrReadOnly, COL_MEAS_PHASE_CH1_3); attrReadOnly->IncRef();
    this->SetColAttr(attrReadOnly, COL_MEAS_PHASE_CH1_4); attrReadOnly->IncRef();
  }
}

SweepTable::~SweepTable()
{
}

void SweepTable::Resize(int size)
{
  wxASSERT(size >= 1);
  if (m_data.size() != size)
  {
    if (m_data.empty())
    {
      m_data.resize(size);
    }
    else
    {
      Measurement meas = m_data.back();
      m_data.resize(size, meas);
    }
    NotifyRowSizeChanged();
  }
}

void SweepTable::FillFrequency(float startFreq, float stopFreq, bool logScale, bool optimize)
{
  float logOfStartFreq = std::log10(startFreq);
  float logOfStopFreq  = std::log10(stopFreq);

  for (size_t i = 0; i < m_data.size(); ++i)
  {
    // Calculate frequency.
    float freq;
    if (logScale)
    {
      freq = std::pow(10.f, logOfStartFreq + (logOfStopFreq - logOfStartFreq) * i / (m_data.size() - 1));
    }
    else
    {
      freq = startFreq + (stopFreq - startFreq) * i / (m_data.size() - 1);
    }

    // Select Time/DIV to sample one period of wave data.
    float period = 1.0f / freq;
    int timeDiv = 8;      // larger than 250Msps
    while (timeDiv < MAX_TIMEDIV_NUM) 
    {
      float sr = OscilloscopeHard::SampleRate(timeDiv);
      float sampleLength = (float)BUF_4K_LEN / sr;
      float num = sampleLength / period;
      if (num > 1.0f) break;
      timeDiv ++;
    }

    // Optimize frequency to make points of sampled single wave integer. 
    if (optimize)
    {
      float sr = OscilloscopeHard::SampleRate(timeDiv);
      float points = sr / freq;
      freq = sr / wxRound(points);
    }

    m_data[i].ddsFreq = freq;
    m_data[i].timeDiv = timeDiv;
  }

  NotifyUpdate();
}

void SweepTable::FillDDSAmplitude(int ddsAmp)
{
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    m_data[i].ddsAmp = ddsAmp;
  }
  NotifyUpdate();
}

void SweepTable::FillDDSOffset(int ddsOffset)
{
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    m_data[i].ddsOffset = ddsOffset;
  }
  NotifyUpdate();
}

void SweepTable::FillVoltDiv(int ch, int voltDiv)
{
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    m_data[i].voltDiv[ch] = voltDiv;
  }
  NotifyUpdate();
}

void SweepTable::FillAverage(int average)
{
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    m_data[i].average = average;
  }
  NotifyUpdate();
}

void SweepTable::OptimizeVoltDiv(bool ddsTerm50Ohm)
{
  float ch1Att = ddsTerm50Ohm ? 0.5f : 1.0f;

  for (size_t i = 0; i < m_data.size(); ++i)
  {
    // CH1 is determined by reference DDS signal amplitude.
    float ch1Amp = m_data[i].ddsAmp * ch1Att / 1000.0f;
    m_data[i].voltDiv[CH1] = SelectVoltDiv(ch1Amp);

    // CH2 - CH4 are determined by target gain characteristics.
    for (int ch = CH2; ch <= CH4; ++ch)
    {
      float gain = std::pow(10.0f, m_data[i].measGain[ch - 1] / 20.0f);
      float estimatedAmp = ch1Amp * gain;
      m_data[i].voltDiv[ch] = SelectVoltDiv(estimatedAmp);
    }
  }
  NotifyUpdate();
}

bool SweepTable::LoadFromFile(wxString const &fileName)
{
  FILE *file = ::fopen(fileName.c_str(), "rt");
  if (NULL == file)
  {
    return false;
  }

  MeasurementArray newData;
  for (size_t i = 0; ; ++i)
  {
    wchar_t line[1024];
    if (NULL == ::fgetws(line, 1024, file)) break;

    if (i != 0)
    {
      // Read after 2nd line.
      Measurement meas;
      wchar_t szTimeDiv[16], szVoltDiv1[16], szVoltDiv2[16], szVoltDiv3[16], szVoltDiv4[16];
      int result
        = ::swscanf(line, L"%f,%d,%d,%[^,],%[^,],%[^,],%[^,],%[^,],%d,%f,%f,%f,%f,%f,%f",
                    &meas.ddsFreq,
                    &meas.ddsAmp,
                    &meas.ddsOffset,
                    szTimeDiv,
                    szVoltDiv1,
                    szVoltDiv2,
                    szVoltDiv3,
                    szVoltDiv4,
                    &meas.average,
                    &meas.measGain[CH1_2],
                    &meas.measGain[CH1_3],
                    &meas.measGain[CH1_4],
                    &meas.measPhase[CH1_2],
                    &meas.measPhase[CH1_3],
                    &meas.measPhase[CH1_4]);
      if (result != 15)
      {
        ::fclose(file);
        return false;
      }
      meas.timeDiv = OscilloscopeHard::TimeDivFromString(szTimeDiv);
      meas.voltDiv[CH1] = OscilloscopeHard::VoltDivFromString(szVoltDiv1);
      meas.voltDiv[CH2] = OscilloscopeHard::VoltDivFromString(szVoltDiv2);
      meas.voltDiv[CH3] = OscilloscopeHard::VoltDivFromString(szVoltDiv3);
      meas.voltDiv[CH4] = OscilloscopeHard::VoltDivFromString(szVoltDiv4);
      newData.push_back(meas);
    }
  }

  ::fclose(file);

  if (newData.size() >= 1)
  {
    m_data.swap(newData);
    NotifyRowSizeChanged();
    return true;
  }
  else
  {
    return false;
  }
}

bool SweepTable::SaveToFile(wxString const &fileName) const
{
  FILE *file = ::fopen(fileName.c_str(), "wt");
  if (NULL == file)
  {
    return false;
  }

  ::fwprintf(file, L"%s\n", g_csvFileHeader.wc_str());
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    Measurement const &meas = m_data[i];
    
    wxString line;
    line += wxString::Format(_T("%.3f,"), meas.ddsFreq);
    line += wxString::Format(_T("%d,"), meas.ddsAmp);
    line += wxString::Format(_T("%d,"), meas.ddsOffset);
    line += OscilloscopeHard::TimeDivToString(meas.timeDiv) + _T(",");
    line += OscilloscopeHard::VoltDivToString(meas.voltDiv[CH1]) + _T(",");
    line += OscilloscopeHard::VoltDivToString(meas.voltDiv[CH2]) + _T(",");
    line += OscilloscopeHard::VoltDivToString(meas.voltDiv[CH3]) + _T(",");
    line += OscilloscopeHard::VoltDivToString(meas.voltDiv[CH4]) + _T(",");
    line += wxString::Format(_T("%d,"), meas.average);
    line += wxString::Format(_T("%.3f,"), meas.measGain[CH1_2]);
    line += wxString::Format(_T("%.3f,"), meas.measGain[CH1_3]);
    line += wxString::Format(_T("%.3f,"), meas.measGain[CH1_4]);
    line += wxString::Format(_T("%.3f,"), meas.measPhase[CH1_2]);
    line += wxString::Format(_T("%.3f,"), meas.measPhase[CH1_3]);
    line += wxString::Format(_T("%.3f"), meas.measPhase[CH1_4]);

    ::fwprintf(file, L"%s\n", line.wc_str());
  }

  ::fclose(file);
  return TRUE;
}

void SweepTable::Draw(wxDC &dc, wxRect const &rect, bool chEnabled[MAX_CH_NUM - 1], bool logFreq, SweepTableDrawOption const &option) const
{
  GainPhaseRenderer renderer(m_data, dc, rect, chEnabled, logFreq);
  renderer.CalcDrawRange();
  renderer.CalcFrameSize();

  renderer.FillBackground();
  renderer.DrawGrid();
  renderer.DrawData();

  if (option.drawCurrentFreq)
  {
    renderer.DrawFreqMarker(option.currentFreq, *wxRED);
  }
}

int SweepTable::GetNumberRows()
{
  return m_data.size();
}

int SweepTable::GetNumberCols()
{
  return COL_MAX;
}

bool SweepTable::IsEmptyCell(int row, int col)
{
  return (row >= 0 && row < (int)m_data.size() && col >= 0 && col < COL_MAX);
}

wxString SweepTable::GetValue(int row, int col)
{
  if (row >= 0 && row < (int)m_data.size() && col >= 0 && col < COL_MAX)
  {
    Measurement &item = m_data[row];
    
    wxString str;
    switch (col)
    {
    case COL_DDS_FREQ:          str = wxString::Format(_T("%.3f"), item.ddsFreq);             break;
    case COL_DDS_AMP:           str = wxString::Format(_T("%d"), item.ddsAmp);                break;
    case COL_DDS_OFFSET:        str = wxString::Format(_T("%d"), item.ddsOffset);             break;
    case COL_TIME_DIV:          str = OscilloscopeHard::TimeDivToString(item.timeDiv);        break;
    case COL_VOLT_DIV_CH1:      str = OscilloscopeHard::VoltDivToString(item.voltDiv[CH1]);   break;
    case COL_VOLT_DIV_CH2:      str = OscilloscopeHard::VoltDivToString(item.voltDiv[CH2]);   break;
    case COL_VOLT_DIV_CH3:      str = OscilloscopeHard::VoltDivToString(item.voltDiv[CH3]);   break;
    case COL_VOLT_DIV_CH4:      str = OscilloscopeHard::VoltDivToString(item.voltDiv[CH4]);   break;
    case COL_AVERAGE:           str = wxString::Format(_T("%d"), item.average);               break;
    case COL_MEAS_GAIN_CH1_2:   str = wxString::Format(_T("%.3f"), item.measGain[CH1_2]);     break;
    case COL_MEAS_GAIN_CH1_3:   str = wxString::Format(_T("%.3f"), item.measGain[CH1_3]);     break;
    case COL_MEAS_GAIN_CH1_4:   str = wxString::Format(_T("%.3f"), item.measGain[CH1_4]);     break;
    case COL_MEAS_PHASE_CH1_2:  str = wxString::Format(_T("%.3f"), item.measPhase[CH1_2]);    break;
    case COL_MEAS_PHASE_CH1_3:  str = wxString::Format(_T("%.3f"), item.measPhase[CH1_3]);    break;
    case COL_MEAS_PHASE_CH1_4:  str = wxString::Format(_T("%.3f"), item.measPhase[CH1_4]);    break;
    }
    return str;
  }
  else
  {
    return _T("");
  }
}

void SweepTable::SetValue(int row, int col, wxString const &value)
{
  if (row >= 0 && row < (int)m_data.size() && col >= 0 && col < COL_MAX)
  {
    Measurement &item = m_data[row];
    
    double dvalue;
    long lvalue;

    switch (col)
    {
    case COL_DDS_FREQ:          if (value.ToDouble(&dvalue)) item.ddsFreq = dvalue;             break;
    case COL_DDS_AMP:           if (value.ToLong(&lvalue)) item.ddsAmp = lvalue;                break;
    case COL_DDS_OFFSET:        if (value.ToLong(&lvalue)) item.ddsOffset = lvalue;             break;
    case COL_TIME_DIV:          item.timeDiv = OscilloscopeHard::TimeDivFromString(value);      break;
    case COL_VOLT_DIV_CH1:      item.voltDiv[CH1] = OscilloscopeHard::VoltDivFromString(value); break;
    case COL_VOLT_DIV_CH2:      item.voltDiv[CH2] = OscilloscopeHard::VoltDivFromString(value); break;
    case COL_VOLT_DIV_CH3:      item.voltDiv[CH3] = OscilloscopeHard::VoltDivFromString(value); break;
    case COL_VOLT_DIV_CH4:      item.voltDiv[CH4] = OscilloscopeHard::VoltDivFromString(value); break;
    case COL_AVERAGE:           if (value.ToLong(&lvalue)) item.average = lvalue;               break;
    case COL_MEAS_GAIN_CH1_2:   if (value.ToDouble(&dvalue)) item.measGain[CH1_2] = dvalue;     break;
    case COL_MEAS_GAIN_CH1_3:   if (value.ToDouble(&dvalue)) item.measGain[CH1_3] = dvalue;     break;
    case COL_MEAS_GAIN_CH1_4:   if (value.ToDouble(&dvalue)) item.measGain[CH1_4] = dvalue;     break;
    case COL_MEAS_PHASE_CH1_2:  if (value.ToDouble(&dvalue)) item.measPhase[CH1_2] = dvalue;    break;
    case COL_MEAS_PHASE_CH1_3:  if (value.ToDouble(&dvalue)) item.measPhase[CH1_3] = dvalue;    break;
    case COL_MEAS_PHASE_CH1_4:  if (value.ToDouble(&dvalue)) item.measPhase[CH1_4] = dvalue;    break;
    }
  }
}

wxString SweepTable::GetColLabelValue(int col)
{
  if (col >= 0 && col < COL_MAX)
  {
    wxString str;
    switch (col)
    {
    case COL_DDS_FREQ:          str = _("DDS Freq.[Hz]");   break;
    case COL_DDS_AMP:           str = _("DDS Ampl.[mV]");   break;
    case COL_DDS_OFFSET:        str = _("DDS Offset[mV]");  break;
    case COL_TIME_DIV:          str = _("Time/DIV");        break;
    case COL_VOLT_DIV_CH1:      str = _("Volt1/DIV");       break;
    case COL_VOLT_DIV_CH2:      str = _("Volt2/DIV");       break;
    case COL_VOLT_DIV_CH3:      str = _("Volt3/DIV");       break;
    case COL_VOLT_DIV_CH4:      str = _("Volt4/DIV");       break;
    case COL_AVERAGE:           str = _("Average");         break;
    case COL_MEAS_GAIN_CH1_2:   str = _("|G12|[dB]");       break;
    case COL_MEAS_GAIN_CH1_3:   str = _("|G13|[dB]");       break;
    case COL_MEAS_GAIN_CH1_4:   str = _("|G14|[dB]");       break;
    case COL_MEAS_PHASE_CH1_2:  str = _("arg(G12)[deg]");   break;
    case COL_MEAS_PHASE_CH1_3:  str = _("arg(G13)[deg]");   break;
    case COL_MEAS_PHASE_CH1_4:  str = _("arg(G14)[deg]");   break;
    }
    return str;
  }
  else
  {
    return _T("");
  }
}

void SweepTable::NotifyUpdate()
{
  wxGrid *grid = GetView();
  if (NULL != grid) grid->Refresh(false);
}

//------------------------------------------------------------------------
void SweepTable::NotifyRowSizeChanged()
{
  wxGrid *grid = GetView();
  if (NULL == grid) return;

  // Suppress repainting of the grid.
  grid->BeginBatch();

  // Clear user selection area.
  grid->ClearSelection();
  
  // Disable Editing of the grid.
  if (grid->IsCellEditControlEnabled())
  {
    grid->DisableCellEditControl();
  }

  // Notify that all data is deleted.
  int gridRows = grid->GetNumberRows();
  wxGridTableMessage pop(this, wxGRIDTABLE_NOTIFY_ROWS_DELETED, 0, gridRows);
  grid->ProcessTableMessage(pop);

  // Notify that new data is set
  int newGridRows = this->GetNumberRows();
  wxGridTableMessage push(this, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, newGridRows);
  grid->ProcessTableMessage(push);

  grid->EndBatch();
}

int SweepTable::SelectVoltDiv(float amplitude)
{
  int voltDiv = 0;
  while (voltDiv < MAX_VOLTDIV_NUM)
  {
    //if (OscilloscopeHard::ValueOfVoltDiv(voltDiv) > amplitude) break;
    if (OscilloscopeHard::ValueOfVoltDiv(voltDiv) * 2 > amplitude) break;
    //if (OscilloscopeHard::ValueOfVoltDiv(voltDiv) * 4 > amplitude) break;
    voltDiv++;
  }
  return voltDiv;
}

//------------------------------------------------------------------------
GainPhaseRenderer::GainPhaseRenderer(MeasurementArray const &data, wxDC &dc,
  wxRect const &rect, bool chEnabled[MAX_CH_NUM - 1], bool logFreq)
  : m_data(data), m_dc(dc), m_rect(rect), m_logFreq(logFreq)
{
  for (int i = 0; i < MAX_CH_NUM - 1; ++i) m_chEnabled[i] = chEnabled[i];
}

void GainPhaseRenderer::CalcDrawRange()
{
  // Decide gain range.
  m_gainMax = -FLT_MAX;
  m_gainMin = FLT_MAX;

  if (m_data.empty() ||
      (!m_chEnabled[CH1_2] && !m_chEnabled[CH1_3] && !m_chEnabled[CH1_3]))
  {
    m_gainMax = 20.0f;
    m_gainMin = -20.0f;
  }
  else
  {
    for (size_t i = 0; i < m_data.size(); ++i)
    {
      for (int ch = CH1_2; ch <= CH1_4; ++ch)
      {
        if (m_chEnabled[ch])
        {
          float gain = m_data[i].measGain[ch];
          if (m_gainMax < gain) m_gainMax = gain;
          if (m_gainMin > gain) m_gainMin = gain;
        }
      }
    }
  }

  m_gainMax = std::ceil(m_gainMax / 20.0f) * 20.0f;
  m_gainMin = std::floor(m_gainMin / 20.0f) * 20.0f;
  if (m_gainMax == m_gainMin)
  {
    m_gainMax += 20.0f;
    m_gainMin -= 20.0f;
  }

  m_gainGrid = 20.0f;
  m_gainSubGrid = 10.0f;

  // Decide phase range.
  m_phaseMax = 180.0f;
  m_phaseMin = -180.0f;
  m_phaseGrid = 180.0f;
  m_phaseSubGrid = 45.0f;

  // Decide frequency range.
  m_freqMax = -FLT_MAX;
  m_freqMin = FLT_MAX;

  if (m_data.empty())
  {
    m_freqMax = 10.0f;
    m_freqMin = 1.0f;
  }
  for (size_t i = 0; i < m_data.size(); ++i)
  {
    float freq = m_data[i].ddsFreq;
    if (m_freqMax < freq) m_freqMax = freq;
    if (m_freqMin > freq) m_freqMin = freq;
  }

  if (m_logFreq)
  {
    m_freqMax = std::pow(10.0f, std::ceil(std::log10(m_freqMax)));
    m_freqMin = std::pow(10.0f, std::floor(std::log10(m_freqMin)));
    if (m_freqMax == m_freqMin)
    {
      m_freqMax *= 10.0f;
      m_freqMin /= 10.0f;
    }
    m_freqGrid = m_freqSubGrid = 0.0f;  // not used.
  }
  else
  {
    float digitFreqMax = std::pow(10.0f, std::floor(std::log10(m_freqMax)));  // ex.) freqMax = 20000 => digitFreqMax = 10000
    float digitFreqMin = std::pow(10.0f, std::floor(std::log10(m_freqMin)));
    float digitFreq = std::max(digitFreqMax, digitFreqMin);
    m_freqMax = std::ceil(m_freqMax / digitFreq) * digitFreq;
    m_freqMin = std::floor(m_freqMin / digitFreq) * digitFreq;
    if (m_freqMax == m_freqMin)
    {
      m_freqMax *= 2.0f;
      m_freqMin = 0.0f;
    }
    m_freqGrid = digitFreq;
    m_freqSubGrid = digitFreq / 10.0f;
  }
}

void GainPhaseRenderer::CalcFrameSize()
{
  // Calc area of gain and phase.
  // +------------------------------------+
  // |                 10                 |
  // |  +------------------------------+  |
  // |70|                              |20|
  // |  |                              |  |
  // |  +------------------------------+  |
  // |                 50                 |
  // |  +------------------------------+  |
  // |70|                              |20|
  // |  |                              |  |
  // |  +------------------------------+  |
  // |                 40                 |
  // +------------------------------------+
  int w = m_rect.width - 70 - 20;
  int h = (m_rect.height - 10 - 50 - 40) / 2;
  m_gainRect = wxRect(m_rect.x + 70, m_rect.y + 10, w, h);
  m_phaseRect = wxRect(m_rect.x + 70, m_rect.y + h + 10 + 50, w, h);
}

void GainPhaseRenderer::FillBackground()
{
  // Fill backgroud.
  m_dc.SetBackgroundMode(wxSOLID);
  m_dc.SetBrush(*wxBLACK_BRUSH);
  m_dc.SetPen(*wxBLACK_PEN);
  m_dc.DrawRectangle(m_rect);
}

void GainPhaseRenderer::DrawGrid()
{
  wxPen gridPen(*wxWHITE, 1, wxPENSTYLE_SOLID);
  wxPen subGridPen(*wxLIGHT_GREY, 1, wxPENSTYLE_DOT);
  m_dc.SetTextForeground(*wxWHITE);
  m_dc.SetBackgroundMode(wxTRANSPARENT);

  // Draw frequency grid.
  if (m_logFreq)
  {
    int logMin = (int)std::log10(m_freqMin);
    int logMax = (int)std::log10(m_freqMax);
    for (int log = logMin; log <= logMax; ++log)
    {
      float freq = std::pow(10.0f, log);
      int x = FreqToDisp(freq);

      // Main grid.
      m_dc.SetPen(gridPen);
      m_dc.DrawLine(x, m_gainRect.y,  x, m_gainRect.y + m_gainRect.height);
      m_dc.DrawLine(x, m_phaseRect.y, x, m_phaseRect.y + m_phaseRect.height);

      //wxString freqLabel = wxString::Format(_T("%.1e"), freq);
      wxString freqLabel = FormatEng(freq);
      m_dc.DrawText(freqLabel, x - 15, m_gainRect.y + m_gainRect.height + 4);
      m_dc.DrawText(freqLabel, x - 15, m_phaseRect.y + m_phaseRect.height + 4);

      // Sub grid.
      if (log < logMax)
      {
        for (int i = 2; i <= 9; ++i)
        {
          int xx = FreqToDisp(freq * i);
          m_dc.SetPen(subGridPen);
          m_dc.DrawLine(xx, m_gainRect.y,  xx, m_gainRect.y + m_gainRect.height);
          m_dc.DrawLine(xx, m_phaseRect.y, xx, m_phaseRect.y + m_phaseRect.height);
        }
      }

      m_dc.DrawText(_("Freq. [Hz]"), m_gainRect.width / 2 - 20 + 50, m_gainRect.y + m_gainRect.height + 20);
      m_dc.DrawText(_("Freq. [Hz]"), m_phaseRect.width / 2 - 20 + 50, m_phaseRect.y + m_phaseRect.height + 20);
    }
  }
  else
  {
    int iMin = wxRound(m_freqMin / m_freqGrid);
    int iMax = wxRound(m_freqMax / m_freqGrid);
    for (int i = iMin; i <= iMax; ++i)
    {
      float freq = m_freqGrid * i;
      int x = FreqToDisp(freq);

      // Main grid.
      m_dc.SetPen(gridPen);
      m_dc.DrawLine(x, m_gainRect.y,  x, m_gainRect.y + m_gainRect.height);
      m_dc.DrawLine(x, m_phaseRect.y, x, m_phaseRect.y + m_phaseRect.height);

      wxString freqLabel = FormatEng(freq);
      m_dc.DrawText(freqLabel, x - 15, m_gainRect.y + m_gainRect.height + 4);
      m_dc.DrawText(freqLabel, x - 15, m_phaseRect.y + m_phaseRect.height + 4);

      // Sub grid.
      if (i < iMax)
      {
        int jMax = wxRound(m_freqGrid / m_freqSubGrid);
        for (int j = 1; j < jMax; ++j)
        {
          int xx = FreqToDisp(freq + m_freqSubGrid * j);
          m_dc.SetPen(subGridPen);
          m_dc.DrawLine(xx, m_gainRect.y,  xx, m_gainRect.y + m_gainRect.height);
          m_dc.DrawLine(xx, m_phaseRect.y, xx, m_phaseRect.y + m_phaseRect.height);
        }
      }
    }
  }

  // Draw gain grid.
  int iMin = wxRound(m_gainMin / m_gainGrid);
  int iMax = wxRound(m_gainMax / m_gainGrid);
  for (int i = iMin; i <= iMax; ++i)
  {
    float gain = m_gainGrid * i;
    int y = GainToDisp(gain);

    // Main grid.
    m_dc.SetPen(gridPen);
    m_dc.DrawLine(m_gainRect.x, y, m_gainRect.x + m_gainRect.width, y);

    wxString gainLabel = wxString::Format(_T("%.0f"), gain);
    m_dc.DrawText(gainLabel, m_gainRect.x - 30, y - 10);

    // Sub grid.
    if (i < iMax)
    {
      int jMax = wxRound(m_gainGrid / m_gainSubGrid);
      for (int j = 1; j < jMax; ++j)
      {
        int yy = GainToDisp(gain + m_gainSubGrid * j);
        m_dc.SetPen(subGridPen);
        m_dc.DrawLine(m_gainRect.x, yy, m_gainRect.x + m_gainRect.width, yy);
      } 
    }
  }
  m_dc.DrawRotatedText(_T("Gain [dB]"), 10, m_gainRect.y + m_gainRect.height / 2 + 30, 90.0f);

  // Draw phase grid.
  iMin = wxRound(m_phaseMin / m_phaseGrid);
  iMax = wxRound(m_phaseMax / m_phaseGrid);
  for (int i = iMin; i <= iMax; ++i)
  {
    float phase = m_phaseGrid * i;
    int y = PhaseToDisp(phase);

    // Main grid.
    m_dc.SetPen(gridPen);
    m_dc.DrawLine(m_phaseRect.x, y, m_phaseRect.x + m_phaseRect.width, y);

    wxString phaseLabel = wxString::Format(_T("%.0f"), phase);
    m_dc.DrawText(phaseLabel, m_phaseRect.x - 30, y - 10);

    // Sub grid.
    if (i < iMax)
    {
      int jMax = wxRound(m_phaseGrid / m_phaseSubGrid);
      for (int j = 1; j < jMax; ++j)
      {
        int yy = PhaseToDisp(phase + m_phaseSubGrid * j);
        m_dc.SetPen(subGridPen);
        m_dc.DrawLine(m_phaseRect.x, yy, m_phaseRect.x + m_phaseRect.width, yy);
      } 
    }
  }
  m_dc.DrawRotatedText(_T("Phase [deg]"), 10, m_phaseRect.y + m_phaseRect.height / 2 + 30, 90.0f);
}

void GainPhaseRenderer::DrawData()
{
  wxColor colors[MAX_CH_NUM - 1];
  colors[CH1_2] = wxColor(0, 255, 255);
  colors[CH1_3] = wxColor(255, 0, 255);
	colors[CH1_4] = wxColor(0, 255, 0);

  for (size_t i = 0; i < m_data.size(); ++i)
  {
    float freq = m_data[i].ddsFreq;
    int x = FreqToDisp(freq);

    for (int ch = CH1_2; ch <= CH1_4; ++ch)
    {
      if (m_chEnabled[ch])
      {
        float gain = m_data[i].measGain[ch];
        float phase = m_data[i].measPhase[ch];
        int yg = GainToDisp(gain);
        int yp = PhaseToDisp(phase);
        wxRect rectg(x - 2, yg - 2, 5, 5);
        wxRect rectp(x - 2, yp - 2, 5, 5);

        wxPen pen(colors[ch]);
        m_dc.SetPen(pen);

        wxBrush brush(colors[ch]);
        m_dc.SetBrush(brush);

        m_dc.DrawRectangle(rectg);
        m_dc.DrawRectangle(rectp);
      }
    }
  }
}

void GainPhaseRenderer::DrawFreqMarker(float freq, wxColor const &color)
{
  wxPen pen(color, 1, wxPENSTYLE_SOLID);
  int xx = FreqToDisp(freq);
  m_dc.SetPen(pen);
  m_dc.DrawLine(xx, m_gainRect.y,  xx, m_gainRect.y + m_gainRect.height);
  m_dc.DrawLine(xx, m_phaseRect.y, xx, m_phaseRect.y + m_phaseRect.height);
}
  
int GainPhaseRenderer::FreqToDisp(float freq) const
{
  if (m_logFreq)
  {
    float logMin = std::log10(m_freqMin);
    float logMax = std::log10(m_freqMax);
    float log = std::log10(freq);
    return wxRound(m_gainRect.x + m_gainRect.width * (log - logMin) / (logMax - logMin));
  }
  else
  {
    return wxRound(m_gainRect.x + m_gainRect.width * (freq - m_freqMin) / (m_freqMax - m_freqMin)); 
  }
}

int GainPhaseRenderer::GainToDisp(float gain) const
{
  return wxRound(m_gainRect.y + m_gainRect.height - m_gainRect.height * (gain - m_gainMin) / (m_gainMax - m_gainMin));
}

int GainPhaseRenderer::PhaseToDisp(float phase) const
{
  return wxRound(m_phaseRect.y + m_phaseRect.height - m_phaseRect.height * (phase - m_phaseMin) / (m_phaseMax - m_phaseMin));
}

//------------------------------------------------------------------------
