#pragma once

//------------------------------------------------------------------------
// Index of Result Array.
static int const CH1_2 = 0;
static int const CH1_3 = 1;
static int const CH1_4 = 2;

// Sweep table column.
static int const COL_DDS_FREQ         = 0;
static int const COL_DDS_AMP          = 1;
static int const COL_DDS_OFFSET       = 2;
static int const COL_TIME_DIV         = 3;
static int const COL_VOLT_DIV_CH1     = 4;
static int const COL_VOLT_DIV_CH2     = 5;
static int const COL_VOLT_DIV_CH3     = 6;
static int const COL_VOLT_DIV_CH4     = 7;
static int const COL_AVERAGE          = 8;
static int const COL_MEAS_GAIN_CH1_2  = 9;
static int const COL_MEAS_GAIN_CH1_3  = 10;
static int const COL_MEAS_GAIN_CH1_4  = 11;
static int const COL_MEAS_PHASE_CH1_2 = 12;
static int const COL_MEAS_PHASE_CH1_3 = 13;
static int const COL_MEAS_PHASE_CH1_4 = 14;
static int const COL_MAX              = 15;

//------------------------------------------------------------------------
/**
 * Condition and Result of each sweep point.
 */
struct Measurement
{
  Measurement();

  // Condition.
  float ddsFreq;                    // DDS frequency.
  int ddsAmp;                       // DDS amplitude.
  int ddsOffset;                    // DDS offset.

  int timeDiv;                      // Oscilloscope Time / DIV.
  int voltDiv[MAX_CH_NUM];          // Oscilloscope Voltage / DIV.

  int average;                      // Count of averaging.

  // Result.
  float measGain[MAX_CH_NUM - 1];   // Measured gain [dB] (CH1 is reference).
  float measPhase[MAX_CH_NUM - 1];  // Measured phase [rad] (CH1 is reference).
};

//------------------------------------------------------------------------
typedef std::vector<Measurement> MeasurementArray;

//------------------------------------------------------------------------
/**
 * Sweep table draw option.
 */
struct SweepTableDrawOption
{
  bool drawCurrentFreq;
  float currentFreq;
};

//------------------------------------------------------------------------
/**
 * Sweep condition and measurement data.
 */
class SweepTable : public wxGridTableBase
{
public:
  SweepTable();
  virtual ~SweepTable();

  /**
   * Resize table.
   */
  void Resize(int size);

  /**
   * Fill DDS frequency and oscilloscope Time/DIV.
   */
  void FillFrequency(float startFreq, float stopFreq, bool logScale, bool optimize);

  /**
   * Fill DDS amplitude.
   */
  void FillDDSAmplitude(int ddsAmp);

  /**
   * Fill DDS offset.
   */
  void FillDDSOffset(int ddsOffset);

  /**
   * Fill oscilloscope Volt/DIV.
   */
  void FillVoltDiv(int ch, int voltDiv);

  /**
   * Fill count of average.
   */
  void FillAverage(int average);

  /**
   * Optimize Volt/DIV by using previous measurement data.
   */
  void OptimizeVoltDiv(bool ddsTerm50Ohm);

  /**
   * Load data from CSV file.
   */
  bool LoadFromFile(wxString const &fileName);

  /**
   * Save data to CSV file.
   */
  bool SaveToFile(wxString const &fileName) const;

  /**
   * Draw data to wxWidgets device context.
   */
  void Draw(wxDC &dc, wxRect const &rect, bool chEnabled[MAX_CH_NUM - 1], bool logFreq, SweepTableDrawOption const &option) const;

  Measurement &operator [] (int index) { return m_data[index]; }
  Measurement const &operator [] (int index) const { return m_data[index]; }

  int GetSize() const { return m_data.size(); }

  virtual int GetNumberRows();
  virtual int GetNumberCols();
  virtual bool IsEmptyCell(int row, int col);
  virtual wxString GetValue(int row, int col);
  virtual void SetValue(int row, int col, wxString const &value);
  virtual wxString GetColLabelValue(int col);

  /**
   * Notify update of table data to wxGrid object.
   */
  void NotifyUpdate();

private:
  void NotifyRowSizeChanged();
  static int SelectVoltDiv(float amplitude);

  MeasurementArray m_data;
};

//------------------------------------------------------------------------
struct GainPhaseRenderer
{
  GainPhaseRenderer(MeasurementArray const &data, wxDC &dc, wxRect const &rect, bool chEnabled[MAX_CH_NUM - 1], bool logFreq);

  void CalcDrawRange();
  void CalcFrameSize();

  void FillBackground();
  void DrawGrid();
  void DrawData();
  void DrawFreqMarker(float freq, wxColor const &color);
  
  int FreqToDisp(float freq) const;
  int GainToDisp(float gain) const;
  int PhaseToDisp(float phase) const;

  MeasurementArray const &m_data;
  wxDC &m_dc;
  wxRect m_rect;
  bool m_chEnabled[MAX_CH_NUM - 1];
  bool m_logFreq;

  float m_freqMin, m_freqMax, m_freqGrid, m_freqSubGrid;
  float m_gainMin, m_gainMax, m_gainGrid, m_gainSubGrid;
  float m_phaseMin, m_phaseMax, m_phaseGrid, m_phaseSubGrid;
  
  wxRect m_gainRect;
  wxRect m_phaseRect;
};

//------------------------------------------------------------------------
